@@ -11,7 +11,10 @@ my $CLASSES = {};
 # the root accessor: returns the whole data structure, all meta classes
 sub classes { $CLASSES }
 
+# returns all attributes for the given class
 sub attributes { $CLASSES->{ $_[1] } }
+
+# returns the meta-data for the given class
 sub class
 { 
     my ($self, $class) = @_;
@@ -76,6 +79,21 @@ sub exists
     return exists $CLASSES->{ $class };
 }
 
+# returns the default value for the given $class/$attr
+sub attr_default($$) {
+    my( $self, $obj, $attr) = @_;
+    my $class = ref $obj;
+
+    my $meta = Coat::Meta->has( $class, $attr );
+
+    my $default = $meta->{'default'};
+    return undef unless defined $default;
+
+    return (ref $default)
+        ? $default->($obj)  # we have a CODE ref
+        : $default;     # we have a plain scalar
+}
+
 # this method looks for the attribute description in the whole hierarchy 
 # of the class, starting by the lowest leaf.
 # returns the description or undef if not found.
@@ -84,7 +102,6 @@ sub has($$$)
 { 
     my ($self, $class, $attribute) = @_;
 
-
     # if the attribute is declared for us, it's ok
     return $CLASSES->{ $class }{ $attribute } if 
         exists $CLASSES->{ $class }{ $attribute };
@@ -149,7 +166,17 @@ sub is_parent
 
 sub family { $CLASSES->{'@!family'}{ $_[1] } }
 
-sub extends 
+sub add_to_family {
+    my ($self, $class, $parent) = @_;
+    
+    # add the parent to the family if not already present
+    if (not grep /^$parent$/, @{$CLASSES->{'@!family'}{ $class }}) {
+        push @{ $CLASSES->{'@!family'}{ $class } }, $parent; 
+    }
+}
+
+sub extends($$$);
+sub extends($$$)
 { 
     my ($self, $class, $parents) = @_;
     $parents = [$parents] unless ref $parents;
@@ -159,21 +186,15 @@ sub extends
         $CLASSES->{'@!family'}{ $class } = [];
      }
     
-
+    # loop on each parent, add it to family and do the same 
+    # with recursion through its family
     foreach my $parent (@$parents) {
-        # make sure we don't inherit twice
-        confess "Class '$class' already inherits from class '$parent'" if 
-            Coat::Meta->is_family( $class, $parent );
-        
         foreach my $ancestor (@{ Coat::Meta->parents( $parent ) }) {
-            push @{ $CLASSES->{'@!family'}{ $class } }, $ancestor 
-                unless grep /^$ancestor$/, 
-                            @{$CLASSES->{'@!family'}{ $class }};
+            Coat::Meta->extends($class, $ancestor);
         }
-        
-        push @{ $CLASSES->{'@!family'}{ $class } }, $parent;
+        # we do it at the end, so we respect the order of ancestry
+        Coat::Meta->add_to_family($class, $parent);
     }
-
 }
 
 sub modifiers
